@@ -36,13 +36,14 @@
 #include <sstream>
 #include <stdlib.h>
 using namespace std;
+using namespace lineak_core_functions;
 
 #include "defaultplugin.h"
 
 /** These are required */
 #define NUM_MACROS 12 
 
-char* default_symbols[NUM_MACROS] = { "EAK_EJECT", "EAK_OPEN_TRAY_SCSI", "EAK_OPEN_TRAY", "EAK_CLOSE_TRAY", "EAK_VOLUP", "EAK_VOLDOWN", "EAK_PCM_UP", "EAK_PCM_DOWN", "EAK_MUTE", "EAK_PCM_MUTE", "EAK_SLEEP", "EAK_SYM" };
+string default_symbols[NUM_MACROS] = { "EAK_EJECT", "EAK_OPEN_TRAY_SCSI", "EAK_OPEN_TRAY", "EAK_CLOSE_TRAY", "EAK_VOLUP", "EAK_VOLDOWN", "EAK_PCM_UP", "EAK_PCM_DOWN", "EAK_MUTE", "EAK_PCM_MUTE", "EAK_SLEEP", "EAK_SYM" };
 macro_info* default_macinfo = NULL;
 identifier_info* idinfo = NULL;
 LConfig* myConfig = NULL;
@@ -52,15 +53,16 @@ bool trayopen = false;
 PluginManager* plugins = NULL;
 string dname = "";
 displayCtrl* default_Display = NULL;
-Xmgr mX;
+//Xmgr *mX = NULL;
 
 extern "C" int initialize(init_info init) {
         verbose = init.verbose;
 	enable = init.global_enable;
 	myConfig = init.config;
         plugins = init.plugins;
-
-
+	msg("Registered");
+        //if (mX == NULL)
+        //   mX = new Xmgr();
 
         return true;
 }
@@ -123,7 +125,7 @@ extern "C" int exec(LObject* imyKey,XEvent xev) {
                                 macroEAK_MUTE(command,USE_PCM);
                         } else if (command.getMacroType() == "EAK_SLEEP" ) { /* Sleep (not implemented yet) */
                                 macroEAK_SLEEP(command);
-                        } else if (command.getMacroType() == "EAK_SYM" ) { /* Key SYM handling, not implemented yet) */
+                        } else if (command.getMacroType() == "EAK_SYM" ) { /* Key SYM handling */
                                 macroEAK_SYM(imyKey, command);
                         } else if (command.isMacro()) { /** If we are a macro, but we can't find one that matches, return false. */
                                 return false;
@@ -155,7 +157,12 @@ extern "C" void cleanup() {
                 delete (idinfo);
                 idinfo = NULL;
         }
-
+/*
+        if (mX != NULL) {
+                delete(mX);
+                mX = NULL;
+        }
+*/
 	if (verbose) cout << "Done cleaning up plugin defaultplugin" << endl;
 }
 extern "C" void initialize_display(displayCtrl *imyDisplay) {
@@ -163,14 +170,22 @@ extern "C" void initialize_display(displayCtrl *imyDisplay) {
         default_Display = imyDisplay;
 }
 void macroEAK_SYM(LObject *obj, LCommand &command) {
-   const vector<string>& args = command.getArgs();
+const vector<string>& args = command.getArgs();
    string macro = command.getMacroType();
    if (args.size() == 1) {
       /** Expecting a KEYSYM string. */
       /** could be something like: "control+alt+S" or "shift+F1" or just "A" 
        *  This is the keysym we want to emit. */
-      cout << "Not finished yet!" << endl;
 
+	   /* This doesn't quite work as expected. What we may need to do is
+	    * to output each key w/ the correct state in order and then release
+	    * them in the proper order. 
+	    *
+	    * I've discovered that with xev if you press the Alt_L key it outputs normally
+	    * but the release event has the modifier state set 0x8. Control is the same
+	    * on the release it has the modifier state set to 0x4 (which is the modifier numeric
+	    * for the control key.) */
+                         
       unsigned int modifiers = 0;
       int index = 0;
       string parsed = "";
@@ -186,95 +201,82 @@ void macroEAK_SYM(LObject *obj, LCommand &command) {
           parsed = symname.substr(index+1,symname.size()-(index+1));
           smodifiers = symname.substr(0,index);
           cout << "modifiers = " << smodifiers << endl;
-	  cout << "symname = " << parsed << endl;
+          cout << "symname = " << parsed << endl;
           modifiers = lineak_core_functions::getModifierNumericValue(smodifiers);
+          cout << "modifiers numerical = " << modifiers << endl;
       }
-      else
+      else {
           parsed = symname;
+          cout << "modifiers = none" << endl;
+	  cout << "symname = " << parsed << endl;
+      }
 
       name = (char *) malloc (parsed.size()+1);
       strcpy (name, parsed.c_str());
       name[parsed.size()] = '\0';
 
-    mykeysym = XStringToKeysym (parsed.c_str());
-    keysym = &mykeysym;
+      mykeysym = XStringToKeysym(parsed.c_str());
+      keysym = &mykeysym;
 
-    if (*keysym == NoSymbol && '0' <= *name && *name <= '9') {
-        char *fmt = "%ld";
-        char *str = name;
-        unsigned long *val = keysym;
-        if (*str == '0') {
-           str++;
-           fmt = "%lo";
-           if (*str == '\0') {
-              *val = 0;
-           }
-           if (*str == 'x' || *str == 'X') {
-              str++;
-              fmt = "%lx";
-           }
-       }
-       if (*val != 0)
-          sscanf (str, fmt, val);
-    }
+      if (*keysym == NoSymbol && '0' <= *name && *name <= '9') {
+         char *fmt = "%ld";
+         char *str = name;
+         unsigned long *val = keysym;
+         if (*str == '0') {
+            str++;
+            fmt = "%lo";
+            if (*str == '\0') {
+               *val = 0;
+            }
+            if (*str == 'x' || *str == 'X') {
+               str++;
+               fmt = "%lx";
+            }
+         }
+         if (*val != 0)
+            sscanf (str, fmt, val);
+      }
 
-   char *displayname = NULL;
-   Display *dpy = NULL;
-   if ((dpy = XOpenDisplay(displayname)) == NULL)
-      cerr << "Could not open the display." << endl;
+      Display *dpy = NULL;
+      if ((dpy = XOpenDisplay(NULL)) == NULL)
+         cerr << "Could not open the display." << endl;
 
-   int dummy1, dummy2, dummy3, dummy4;
-   if (XTestQueryExtension( dpy, &dummy1, &dummy2, &dummy3, &dummy4 ) == True)
-      cerr << "No Test extension available!" << endl;
-   XTestFakeKeyEvent( dpy, XKeysymToKeycode(dpy,mykeysym), True, CurrentTime );
-   XTestFakeKeyEvent( dpy, XKeysymToKeycode(dpy,mykeysym), False, CurrentTime );
+      XKeyEvent xev;
+        
+      // Get InputFocus window. Of some reason "InputFocus" dosn't work 
+      Window w;
+      int revertTo;
+      XGetInputFocus(dpy, &w, &revertTo);
+  
+      xev.display = dpy; 
+      xev.root = DefaultRootWindow(dpy);
+      xev.window = w;
+      xev.same_screen = True;
+      xev.subwindow = None;
+      xev.type = KeyPress;
+      xev.keycode = XKeysymToKeycode(dpy, mykeysym);
+      //xev.keycode = 56;
+      //xev.state = modifiers+0x10;
+      xev.state = modifiers;
+      //xev.state = 0;
+      xev.time = CurrentTime;
+      xev.x = 1;
+      xev.y = 1;
+      xev.x_root = 1;
+      xev.y_root = 1;
+        
+      if (XSendEvent(dpy, w, True, KeyPressMask, ( XEvent* )&xev) == 0)
+         cerr << "XSendEvent failed." << endl;
 
-   XKeyEvent xev;
-//   XSetWindowAttributes attr;
-//   Window w = 0x5a00001;
-   Window w = InputFocus;
-   xev.type = KeyPress;
-//   xev.state = modifiers;
-   xev.state = 0;
-   xev.display = dpy; 
-   //xev.window = DefaultRootWindow(dpy);
-   xev.window = w;
-   xev.root = DefaultRootWindow(dpy);
-   xev.subwindow = None;
-   xev.time = CurrentTime;
-   xev.x = 0;
-   xev.y = 0;
-   xev.x_root = 0;
-   xev.y_root = 0;
-   xev.keycode = XKeysymToKeycode(dpy,mykeysym);
-   xev.same_screen = True;
+      xev.type = KeyRelease;
+      if (XSendEvent(dpy, w, True, KeyReleaseMask, ( XEvent* )&xev) == 0)
+         cerr << "XSendEvent failed." << endl;
 
-    if (XSendEvent(dpy, w, True, KeyPressMask, ( XEvent* )&xev) == 0)
-       cerr << "XSendEvent failed." << endl;
-
-    XKeyEvent ev;
-    ev.type = KeyRelease;  // is this actually really needed ??
-    ev.display = dpy;
-    ev.window = w;
-    ev.root = DefaultRootWindow(dpy);
-    ev.subwindow = None;
-    ev.time = CurrentTime;
-    ev.x = 0;
-    ev.y = 0;
-    ev.x_root = 0;
-    ev.y_root = 0;
-    ev.state = 0;
-    ev.keycode = XKeysymToKeycode(dpy,mykeysym);
-    ev.same_screen = True;
-    if (XSendEvent( dpy, w, True, KeyReleaseMask, ( XEvent* )&ev ) == 0)
-       cerr << "XSendEvent failed." << endl;
-
-   XTestFakeKeyEvent( dpy, XKeysymToKeycode(dpy,mykeysym), True, CurrentTime );
-   XTestFakeKeyEvent( dpy, XKeysymToKeycode(dpy,mykeysym), False, CurrentTime );
-
-
-   }
+      
+      XSync(dpy, false);
+  }
 }
+
 void macroEAK_OPEN_TRAY(LCommand &command) {
         const vector<string>& args = command.getArgs();
   /** If there is just the macro, and no arguments, use the globally defined cdrom device, and
